# chrono - agendas system
# Copyright (C) 2022  Entr'ouvert
#
# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU Affero General Public License as published
# by the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import datetime
import json

from django.core.exceptions import PermissionDenied
from django.http import HttpResponse, HttpResponseBadRequest
from django.urls import reverse
from django.views.generic import CreateView, DeleteView, DetailView, ListView, UpdateView

from chrono.pricing.forms import CriteriaForm, NewCriteriaForm
from chrono.pricing.models import Criteria, CriteriaCategory


class CriteriaListView(ListView):
    template_name = 'chrono/pricing/manager_criteria_list.html'
    model = CriteriaCategory

    def dispatch(self, request, *args, **kwargs):
        if not request.user.is_staff:
            raise PermissionDenied()
        return super().dispatch(request, *args, **kwargs)

    def get_queryset(self):
        return CriteriaCategory.objects.prefetch_related('criterias')


criteria_list = CriteriaListView.as_view()


class CriteriaCategoryAddView(CreateView):
    template_name = 'chrono/pricing/manager_criteria_category_form.html'
    model = CriteriaCategory
    fields = ['label']

    def dispatch(self, request, *args, **kwargs):
        if not request.user.is_staff:
            raise PermissionDenied()
        return super().dispatch(request, *args, **kwargs)

    def get_success_url(self):
        return reverse('chrono-manager-pricing-criteria-list')


criteria_category_add = CriteriaCategoryAddView.as_view()


class CriteriaCategoryEditView(UpdateView):
    template_name = 'chrono/pricing/manager_criteria_category_form.html'
    model = CriteriaCategory
    fields = ['label', 'slug']

    def dispatch(self, request, *args, **kwargs):
        if not request.user.is_staff:
            raise PermissionDenied()
        return super().dispatch(request, *args, **kwargs)

    def get_success_url(self):
        return reverse('chrono-manager-pricing-criteria-list')


criteria_category_edit = CriteriaCategoryEditView.as_view()


class CriteriaCategoryDeleteView(DeleteView):
    template_name = 'chrono/manager_confirm_delete.html'
    model = CriteriaCategory

    def dispatch(self, request, *args, **kwargs):
        if not request.user.is_staff:
            raise PermissionDenied()
        return super().dispatch(request, *args, **kwargs)

    def get_success_url(self):
        return reverse('chrono-manager-pricing-criteria-list')


criteria_category_delete = CriteriaCategoryDeleteView.as_view()


class CriteriaCategoryExport(DetailView):
    model = CriteriaCategory

    def dispatch(self, request, *args, **kwargs):
        if not request.user.is_staff:
            raise PermissionDenied()
        return super().dispatch(request, *args, **kwargs)

    def get(self, request, *args, **kwargs):
        response = HttpResponse(content_type='application/json')
        today = datetime.date.today()
        attachment = 'attachment; filename="export_pricing_category_{}_{}.json"'.format(
            self.get_object().slug, today.strftime('%Y%m%d')
        )
        response['Content-Disposition'] = attachment
        json.dump({'pricing_categories': [self.get_object().export_json()]}, response, indent=2)
        return response


criteria_category_export = CriteriaCategoryExport.as_view()


class CriteriaOrder(DetailView):
    model = CriteriaCategory

    def dispatch(self, request, *args, **kwargs):
        if not request.user.is_staff:
            raise PermissionDenied()
        return super().dispatch(request, *args, **kwargs)

    def get(self, request, *args, **kwargs):
        if 'new-order' not in request.GET:
            return HttpResponseBadRequest('missing new-order parameter')
        category = self.get_object()
        try:
            new_order = [int(x) for x in request.GET['new-order'].split(',')]
        except ValueError:
            return HttpResponseBadRequest('incorrect new-order parameter')
        criterias = category.criterias.all()
        if set(new_order) != {x.pk for x in criterias} or len(new_order) != len(criterias):
            return HttpResponseBadRequest('incorrect new-order parameter')
        criterias_by_id = {c.pk: c for c in criterias}
        for i, c_id in enumerate(new_order):
            criterias_by_id[c_id].order = i + 1
            criterias_by_id[c_id].save()
        return HttpResponse(status=204)


criteria_order = CriteriaOrder.as_view()


class CriteriaAddView(CreateView):
    template_name = 'chrono/pricing/manager_criteria_form.html'
    model = Criteria
    form_class = NewCriteriaForm

    def dispatch(self, request, *args, **kwargs):
        self.category_pk = kwargs.pop('category_pk')
        if not request.user.is_staff:
            raise PermissionDenied()
        return super().dispatch(request, *args, **kwargs)

    def get_form_kwargs(self):
        kwargs = super().get_form_kwargs()
        if not kwargs.get('instance'):
            kwargs['instance'] = self.model()
        kwargs['instance'].category_id = self.category_pk
        return kwargs

    def get_success_url(self):
        return reverse('chrono-manager-pricing-criteria-list')


criteria_add = CriteriaAddView.as_view()


class CriteriaEditView(UpdateView):
    template_name = 'chrono/pricing/manager_criteria_form.html'
    model = Criteria
    form_class = CriteriaForm

    def dispatch(self, request, *args, **kwargs):
        self.category_pk = kwargs.pop('category_pk')
        if not request.user.is_staff:
            raise PermissionDenied()
        return super().dispatch(request, *args, **kwargs)

    def get_queryset(self):
        return Criteria.objects.filter(category=self.category_pk)

    def get_success_url(self):
        return reverse('chrono-manager-pricing-criteria-list')


criteria_edit = CriteriaEditView.as_view()


class CriteriaDeleteView(DeleteView):
    template_name = 'chrono/manager_confirm_delete.html'
    model = Criteria

    def dispatch(self, request, *args, **kwargs):
        self.category_pk = kwargs.pop('category_pk')
        if not request.user.is_staff:
            raise PermissionDenied()
        return super().dispatch(request, *args, **kwargs)

    def get_queryset(self):
        return Criteria.objects.filter(category=self.category_pk)

    def get_success_url(self):
        return reverse('chrono-manager-pricing-criteria-list')


criteria_delete = CriteriaDeleteView.as_view()
